﻿// Marshall Systems
// 
// Solution: infgrid
// Project: infgrid
// Filename: Form1.cs
// Created: 2015-05-29 6:18 AM
// Modified: 2015-05-29 7:32 AM
// 
// Comments:
// 

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

using Newtonsoft.Json;

using Parts.Model;

using AutoCompleteMode = Infragistics.Win.AutoCompleteMode;

namespace infgrid
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            setupHandlers();
            populateGrid();
            populateComboFrom(getPartsList());
        }

        private static IEnumerable<ListItem> getPartsList()
        {
            // As I do not have prts.json file I cannot run this code
            //string path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "parts.json");
            //string partsJson = File.ReadAllText(path);
            //return JsonConvert.DeserializeObject<IList<ListItem>>(partsJson);

            // 10 000 List Items added
            List<ListItem> parts = new List<ListItem>();
            for (int index = 0; index < 10000; index++)
            {
                string description = string.Format("Item #{0}", index);
                int id = index;
                object tag = null;
                ListItem item = new ListItem()
                {
                    Description = description,
                    Id = id,
                    Tag = tag,
                };

                parts.Add(item);
            }

            return parts;
        }

        private static void onGridInitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.GroupByBox.Hidden = true;
            e.Layout.TabNavigation = TabNavigation.NextControlOnLastCell;

            // Resize columns to width of grid
            e.Layout.AutoFitStyle = AutoFitStyle.ResizeAllColumns;

            e.Layout.Override.CellClickAction = CellClickAction.Edit;
            //Don’t display expansion indicators (+ signs to the left) for nodes in a multi-band grid that have no descendants:
            e.Layout.Override.ExpansionIndicator = ShowExpansionIndicator.CheckOnDisplay;
            e.Layout.Override.AllowAddNew = AllowAddNew.FixedAddRowOnTop;
            e.Layout.Override.AllowUpdate = DefaultableBoolean.True;
            e.Layout.Override.AllowDelete = DefaultableBoolean.True;

            UltraGridBand bandKit = e.Layout.Bands[0];
            UltraGridBand bandParts = e.Layout.Bands[1];

            // Make row selectors visible
            bandKit.Override.RowSelectors = DefaultableBoolean.True;
            // Customize the appearance of the row selectors.
            e.Layout.Override.RowSelectorAppearance.BackColor = Color.Cyan;

            bandParts.Override.RowSelectors = DefaultableBoolean.True;
            // Hide columns parent band
            bandKit.Columns["Id"].Hidden = true;

            // Hide columns child band
            bandParts.Columns["Id"].Hidden = true;
            bandParts.Columns["KitId"].Hidden = true;

            // Set Column descriptions, widths & appearances for child band
            bandParts.Columns["PartId"].Width = 450;
            //         band.Columns["PartId"].SortIndicator = SortIndicator.Ascending;
            bandParts.Columns["PartId"].Header.Caption = "PE Part Number";

            bandParts.Columns["Quantity"].CellAppearance.TextHAlign = HAlign.Right;
            bandParts.Columns["Quantity"].Format = "###,##0.00";
            bandParts.Columns["Quantity"].MaskInput = "999999.99";
            bandParts.Columns["Quantity"].Width = 70;
        }

        private void onInitializeLayoutComboPartList(object sender, InitializeLayoutEventArgs e)
        {
            UltraCombo combo = sender as UltraCombo;
            if (combo != null)
            {
                int widthDescriptionColumnInGrid = grdKit.DisplayLayout.Bands[1].Columns["PartId"].Width;
                e.Layout.Bands[0].Columns["Description"].Width = widthDescriptionColumnInGrid - 20;
                e.Layout.Bands[0].Columns["Id"].Hidden = true;
                e.Layout.Bands[0].Columns["Tag"].Hidden = true;

                combo.ValueMember = "Id";
                combo.DisplayMember = "Description";

                // This is not taken into account for the grid column. You need to set this at the grid column in order to work
                combo.AutoCompleteMode = AutoCompleteMode.SuggestAppend;
                combo.AutoSuggestFilterMode = AutoSuggestFilterMode.Contains;
                combo.DropDownStyle = UltraComboStyle.DropDown;
            }
        }

        private void populateGrid()
        {
            grdKit.DataSource = new List<Kit>
         {
            new Kit
            {
               Id = 1,
               Description = "Parent",
               KitParts = new BindingList<KitPart> {new KitPart {Id = 1, KitId = 1, PartId = 1, Quantity = 10}}
            }
         };
        }

        private void populateComboFrom(IEnumerable<ListItem> partList)
        {
            UltraCombo combo = new UltraCombo();
            combo.InitializeLayout += onInitializeLayoutComboPartList;
            combo.DataSource = new BindingList<ListItem>(partList.ToList());
            grdKit.DisplayLayout.Bands[1].Columns["PartId"].EditorComponent = combo;

            // You need to set drop down style here on the grid column level in order to make it work
            grdKit.DisplayLayout.Bands[1].Columns["PartId"].AutoCompleteMode = AutoCompleteMode.SuggestAppend;
            grdKit.DisplayLayout.Bands[1].Columns["PartId"].AutoSuggestFilterMode = AutoSuggestFilterMode.Contains;
            grdKit.DisplayLayout.Bands[1].Columns["PartId"].Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDown;
        }

        private void setupHandlers()
        {
            grdKit.InitializeLayout += onGridInitializeLayout;
        }
    }
}